/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: callbacks.c,v 1.11 2002/10/07 11:55:12 dolphy Exp $ */
 
#include "getcontrol.h"

/* Callbacks */

gboolean
getcontrol_quit_callback (GtkWidget * widget, gpointer data)
{	
	if (select_timeout_id) /* Removing selection timer */
		gtk_timeout_remove (select_timeout_id);
	
	select_timeout_id = 0;
	
	if (screen_saver_timeout_id) /* Removing screen saver timer */
		gtk_timeout_remove (screen_saver_timeout_id);
	
	screen_saver_timeout_id = 0;
		
	if (draw_timeout_id) /* Removing animation timer */
		gtk_timeout_remove (draw_timeout_id);
	
	draw_timeout_id = 0;
	
	if (ss_draw_timeout_id)
		gtk_timeout_remove (ss_draw_timeout_id);
	
	ss_draw_timeout_id = 0;
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	if (background_orig)
		gdk_pixbuf_unref (background_orig);
	
	free_images_list ();
	gtk_main_quit ();
}


gboolean
getcontrol_about_callback (GtkWidget * widget, gpointer data)
{
	GdkPixbuf *pixbuf = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {
		"Julien MOUTTE. <julien@moutte.net>",
		"Matthias SAOU. <matthias.saou@est.une.marmotte.net>",
		NULL
	};

	/* Documenters credits */

	gchar *documenters[] = {
		NULL
	};

	/* Translator credits */

	gchar *translator_credits = _("translator_credits");
	
	if (about) {
		gtk_window_present (GTK_WINDOW (about));
		return;
	}

	{
		char *filename = NULL;
		filename = gnome_program_locate_file (
				NULL,
				GNOME_FILE_DOMAIN_APP_PIXMAP,
				("getcontrol.png"),
				TRUE, NULL);

		if (filename != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
			g_free (filename);
		}
	}

	about = gnome_about_new (
			_("Getcontrol"), VERSION,
			"(C) 2002 the Free Software Foundation",
			_("The front end you need for multimedia boxes."),
			(const char **)authors,
			(const char **)documenters,
			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
			pixbuf
				);

	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
	gtk_widget_show (about);
	g_signal_connect (
			G_OBJECT (about), "destroy",
			G_CALLBACK (gtk_widget_destroyed), &about);

	return TRUE;
}

/* Expose callback for the drawing area */
gboolean
expose_callback (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	guchar *pixels;
	int rowstride;

	rowstride = gdk_pixbuf_get_rowstride (frame);

	pixels = gdk_pixbuf_get_pixels (frame) + rowstride * event->area.y + event->area.x * 3;

	gdk_draw_rgb_image_dithalign (
			widget->window,
			widget->style->black_gc,
			event->area.x, event->area.y,
			event->area.width, event->area.height,
			GDK_RGB_DITHER_NORMAL,
			pixels, rowstride,
			event->area.x, event->area.y);

	return TRUE;
}

/* Callback for key_press_event management */
gboolean
key_pressed_callback (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
	
	switch (current_status) {
		case IDLE :		
			switch (event->keyval) {
				case 65363 :
					direction = RIGHT;
					break;;
				case 65361 :
					direction = LEFT;
					break;;
				case 32 :
					if (fullscreen)
						fullscreen = FALSE;
					else
						fullscreen = TRUE;
					
					/* Storing the change in GConf */
	
					gconf_client_set_bool (conf_client,
							FULLSCREEN_PATH, fullscreen, NULL);
					
					return TRUE;
				default :
					return TRUE;;
			}
			
			if (select_timeout_id) { /* User want to rotate more */
		
				gtk_timeout_remove (select_timeout_id);
			
				select_timeout_id = 0;
				
			}
	
			menu_rotation_manager (NULL);
			break;;
		case SELECTED : /* Key pressed while selected checking if we validate or
						not and disabling screen saver */
			if (screen_saver_timeout_id) {
				gtk_timeout_remove (screen_saver_timeout_id);
				screen_saver_timeout_id = 0;
			}
			switch (event->keyval) {
				case 65363 :
					direction = RIGHT;
					menu_deselection_manager (NULL);
					break;;
				case 65361 :
					direction = LEFT;
					menu_deselection_manager (NULL);
					break;;
				case 32 :
					if (fullscreen)
						fullscreen = FALSE;
					else
						fullscreen = TRUE;
					
					/* Storing the change in GConf */
	
					gconf_client_set_bool (conf_client,
							FULLSCREEN_PATH, fullscreen, NULL);
					
					return TRUE;
				default :
					direction = LEFT;
					menu_deselection_manager (NULL);
			}
			break;;
		case SCREEN_SAVING :
			
			if (ss_draw_timeout_id) {
				gtk_timeout_remove (ss_draw_timeout_id);
				ss_draw_timeout_id = 0;
			}
			
			draw_menu();
			
			if (!select_timeout_id) /* Installing timer for selection */
				select_timeout_id = gtk_timeout_add (selection_delay, menu_selection_manager, NULL);
		
			if (!screen_saver_timeout_id) /* Installing timer for screen saver */
				screen_saver_timeout_id = gtk_timeout_add (screensaver_delay, screen_saver_manager, NULL);
			
			break;;
	}

	
	return TRUE;
}

/* Callback for resizing and first drawing */
gboolean
configure_callback (GtkWidget * widget, GdkEventConfigure *event)
{

	gdouble width_ratio_scale, height_ratio_scale;
	
	remove_all_timers ();
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	
	draw_width = widget->allocation.width;
	draw_height = widget->allocation.height;

	/* Creating our pixbufs */
	
	frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
		
	frame_backup = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);

	background = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
	
	/* Middle of the picture */
	
	xmid = draw_width / 2.0;
	ymid = draw_height / 2.0;

	/* Calculating radius */

	radius = MIN (xmid, ymid) / 2.0;
	
	width_ratio_scale = (gdouble) draw_width / (gdouble) back_width;
	height_ratio_scale = (gdouble) draw_height / (gdouble) back_height;
	
	/* Drawing & scaling our background */
	
	gdk_pixbuf_composite (
		background_orig,
		background,
		0, 0,
		draw_width, draw_height,
		0, 0,
		width_ratio_scale, height_ratio_scale,
		GDK_INTERP_BILINEAR,
		255);

	/* Drawing menu */
	
	draw_menu ();
	
	if (!select_timeout_id) /* Installing timer for selection */
		select_timeout_id = gtk_timeout_add (selection_delay, menu_selection_manager, NULL);
		
	if (!screen_saver_timeout_id) /* Installing timer for screen saver */
					screen_saver_timeout_id = gtk_timeout_add (screensaver_delay, screen_saver_manager, NULL);
	
	return TRUE;
}

gboolean
remove_all_timers ()
{
	if (select_timeout_id) /* Removing selection timer */
		gtk_timeout_remove (select_timeout_id);
	
	select_timeout_id = 0;
	
	if (screen_saver_timeout_id) /* Removing screen saver timer */
		gtk_timeout_remove (screen_saver_timeout_id);
	
	screen_saver_timeout_id = 0;
		
	if (draw_timeout_id) /* Removing animation timer */
		gtk_timeout_remove (draw_timeout_id);
	
	draw_timeout_id = 0;
	
	if (ss_draw_timeout_id)
		gtk_timeout_remove (ss_draw_timeout_id);
	
	ss_draw_timeout_id = 0;
	
	return TRUE;
}


/* This is called when our timer keys are changed. */
void
gconf_notify_global (GConfClient *client,
		   guint        cnxn_id,
		   GConfEntry  *entry,
		   gpointer     user_data)
{
	/* Analysing the changed key */
	
	if ( strcmp("/apps/getcontrol/global/anim_frame_delay", gconf_entry_get_key (entry) ) == 0) {
		
		animation_frame_delay = gconf_client_get_int (conf_client, gconf_entry_get_key (entry), NULL);
		
		/* Our animation frame delay has been changed */
		
		if (draw_timeout_id) {
			
			gtk_timeout_remove (draw_timeout_id); /* Removing animation timer */
			
			draw_timeout_id = 0;
			
			switch (current_status) { /* Reenabling the correct animation timer */
				case ROTATING :
					draw_timeout_id = gtk_timeout_add (animation_frame_delay, menu_rotation_manager, NULL);
					break;;
				case SELECTING :
					draw_timeout_id = gtk_timeout_add (animation_frame_delay, menu_selection_manager, NULL);
					break;;
				case DESELECTING :
					draw_timeout_id = gtk_timeout_add (animation_frame_delay, menu_deselection_manager, NULL);
					break;;
				default :
					break;;
			}
			
		}
		
		if (ss_draw_timeout_id) {
			
			gtk_timeout_remove (ss_draw_timeout_id);
			
			ss_draw_timeout_id = gtk_timeout_add (animation_frame_delay, screen_saver_manager, NULL);
		}
	}
	
	if ( strcmp("/apps/getcontrol/global/screensaver_delay", gconf_entry_get_key (entry) ) == 0) {
		
		screensaver_delay = gconf_client_get_int (conf_client, gconf_entry_get_key (entry), NULL);
		
		if (screen_saver_timeout_id) {
			
			gtk_timeout_remove (screen_saver_timeout_id);
			
			screen_saver_timeout_id = gtk_timeout_add (screensaver_delay, screen_saver_manager, NULL);
			
		}
	}
	
	if ( strcmp("/apps/getcontrol/global/selection_delay", gconf_entry_get_key (entry) ) == 0) {
		
		selection_delay = gconf_client_get_int (conf_client, gconf_entry_get_key (entry), NULL);
		
		if (select_timeout_id) {
			
			gtk_timeout_remove (select_timeout_id);
			
			select_timeout_id = gtk_timeout_add (selection_delay, menu_selection_manager, NULL);
			
		}
	}
	
	if ( strcmp(FULLSCREEN_PATH, gconf_entry_get_key (entry) ) == 0) {
		
		fullscreen = gconf_client_get_bool (conf_client, FULLSCREEN_PATH, NULL);

		remove_all_timers ();
		
		toggle_fullscreen ();
	}
	
	
}

/* This is called when our images keys are changed. */
void
gconf_notify_imgs (GConfClient *client,
		   guint        cnxn_id,
		   GConfEntry  *entry,
		   gpointer     user_data)
{
	GError * error = NULL;
	image_node * draw_images_list = NULL;
	gdouble width_ratio_scale, height_ratio_scale;

	if (strcmp("/apps/getcontrol/images/bg_path", gconf_entry_get_key (entry)) == 0 ) {
	
		/* Our background picture has been changed */
		
		if (background_path)
			g_free (background_path);
	
		/* building background filename */
	
		background_path = g_strdup_printf ("%s/%s", PIXMAPDIR,
							gconf_client_get_string (conf_client,
							"/apps/getcontrol/images/bg_path", NULL));
	
		if (background_orig)
			gdk_pixbuf_unref (background_orig);
	
		/* Loading the JPEG background image */

		background_orig = gdk_pixbuf_new_from_file ( background_path, &error);
	
		if (!background_orig) {
			g_message ("failed to load background");
			return;
		}
		
		/* Getting background size */

		back_width = gdk_pixbuf_get_width (background_orig);
		back_height = gdk_pixbuf_get_height (background_orig);
		
		/* Middle of the picture */
	
		xmid = back_width / 2.0;
		ymid = back_height / 2.0;

		/* Calculating radius */

		radius = MIN (xmid, ymid) / 2.0;
		
		
		width_ratio_scale = (gdouble) draw_width / (gdouble) back_width;
		height_ratio_scale = (gdouble) draw_height / (gdouble) back_height;
	
		/* Drawing & scaling our background */
	
		gdk_pixbuf_composite (
			background_orig,
			background,
			0, 0,
			draw_width, draw_height,
			0, 0,
			width_ratio_scale, height_ratio_scale,
			GDK_INTERP_BILINEAR,
			255);
		
	}
	else {	
		/* Copying the head pointer of the images list */
	
		draw_images_list = images_list;
	
		if (!draw_images_list) /* Images list not initialized */
			return;

		while (draw_images_list) {
			if ( strcmp(draw_images_list->gconf_key, gconf_entry_get_key (entry) ) == 0) {
			
				if (draw_images_list->image) /* Freeing strings */
					gdk_pixbuf_unref(draw_images_list->image);
			
				if (draw_images_list->image_path)
					g_free(draw_images_list->image_path);
			
				draw_images_list->image_path = g_strdup_printf ("%s/%s", PIXMAPDIR,
							gconf_client_get_string (conf_client,
								draw_images_list->gconf_key, NULL));
				
				draw_images_list->image = gdk_pixbuf_new_from_file (draw_images_list->image_path, &error);
		
				if (!draw_images_list->image) {
					g_message ("failed to load image %s", draw_images_list->image_path);
					return;
				}
		
				draw_images_list->width = gdk_pixbuf_get_width (draw_images_list->image);
		
				draw_images_list->height = gdk_pixbuf_get_height (draw_images_list->image);
		
		
			}
		
			draw_images_list = draw_images_list->next_image;
		}
	
	}
}


